package com.mobisoft.youtubedatareading;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

import android.app.Activity;
import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.google.gson.Gson;
import com.mobisoft.youtubedatareading.interfaces.ApiCallAsyncTaskDelegate;
import com.mobisoft.youtubedatareading.interfaces.ImageLoadAsyncTaskDelegate;
import com.mobisoft.youtubedatareading.interfaces.KeyValuePairInterface;
import com.mobisoft.youtubedatareading.interfaces.SearchClickDelegate;
import com.mobisoft.youtubedatareading.interfaces.VideoListItemClickListener;
import com.mobisoft.youtubedatareading.interfaces.YouTubeVideoSearchDelegate;
import com.mobisoft.youtubedatareading.listeners.SearchBtnClickListner;
import com.mobisoft.youtubedatareading.listeners.YouTubeSearchListener;
import com.mobisoft.youtubedatareading.models.VideoListModel;
import com.mobisoft.youtubedatareading.models.YouTubeError;
import com.mobisoft.youtubedatareading.models.YouTubeItems;
import com.mobisoft.youtubedatareading.models.YouTubeModelForJson;
import com.mobisoft.youtubedatareading.network.APICall;
import com.mobisoft.youtubedatareading.network.AbstractModel;
import com.mobisoft.youtubedatareading.network.ApiCallAsyncTask;
import com.mobisoft.youtubedatareading.network.ImageLoadAsynkTask;
import com.mobisoft.youtubedatareading.utils.Logger;
import com.mobisoft.youtubedatareading.utils.UtilityClass;
import com.mobisoft.youtubedatareading.utils.VideoListAdapter;

import eu.erikw.PullToRefreshListView;
import eu.erikw.PullToRefreshListView.OnRefreshListener;

public class YouTubeVideoListActivity extends Activity implements
		KeyValuePairInterface, ApiCallAsyncTaskDelegate,
		ImageLoadAsyncTaskDelegate, SearchClickDelegate,
		YouTubeVideoSearchDelegate {
	private Context context;
	private ArrayList<VideoListModel> youtubeVedioList = new ArrayList<VideoListModel>();
	private VideoListAdapter videoListAdaptar;
	private PullToRefreshListView listYoutubeVideos;
	private YouTubeModelForJson youTubeData;
	private String url;
	private String categoryUrl;
	private int startIndex;
	private int maxResult;
	private Button btnMore;
	private CustomProgressDialog progressDialog;

	private int i = 0;
	private ArrayList<String> thumbnailArrayList = new ArrayList<String>();

	private APICall apiCall;
	private AbstractModel requestModel;
	private String query;
	private TextView txtHeading;
	private TextView txtTitle;
	private ImageButton imgBtnSearch;
	private ImageButton imgBtnSearchInspirations;
	private RelativeLayout rltLayoutSearch;
	private EditText etSearch;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.videos_list);

		Bundle bundle = getIntent().getExtras();

		if (bundle != null) {
			url = bundle.getString(URL);
			query = bundle.getString(SEARCH_KEYWORD);
		}
		initLayout();
		Logger.debug(url);

		btnMore.setOnClickListener(new BtnMoreOnClicklistener());

		requestModel.setRequestUrl(url + query);
		progressDialog = new CustomProgressDialog(context, "YouTube",
				"Fetchin videos list,please wait...");
		progressDialog.setCancelable(false);
		progressDialog.show();

		new ApiCallAsyncTask(YouTubeVideoListActivity.this, null, apiCall,
				requestModel, null, "", "", GET_METHOD).execute((Void) null);

	}

	private class BtnMoreOnClicklistener implements OnClickListener {

		@Override
		public void onClick(View v) {
			listYoutubeVideos.removeFooterView(btnMore);
			url = categoryUrl + START_INDEX + startIndex + MAX_RESULT
					+ maxResult + URL_END_TAG + query;

			progressDialog = new CustomProgressDialog(context, "YouTube",
					"Fetchin videos list,please wait...");
			progressDialog.setCancelable(false);
			progressDialog.show();

			apiCall = new APICall();
			requestModel = new AbstractModel();
			requestModel.setRequestUrl(url);

			new ApiCallAsyncTask(YouTubeVideoListActivity.this, null, apiCall,
					requestModel, null, "", "", GET_METHOD)
					.execute((Void) null);

		}

	}

	private void initLayout() {
		context = this;
		startIndex = 1;
		maxResult = 20;
		apiCall = new APICall();
		requestModel = new AbstractModel();

		Bundle bundle = getIntent().getExtras();

		if (bundle != null) {
			categoryUrl = bundle.getString(URL);
			query = bundle.getString(SEARCH_KEYWORD);

		} else {
			categoryUrl = YOUTUBE_BROWSE_URL;
		}
		url = categoryUrl + START_INDEX + startIndex + MAX_RESULT + maxResult
				+ URL_END_TAG + query;

		Logger.debug("Youtube URL:" + url);

		txtHeading = (TextView) findViewById(R.id.txt_heading_video_list);
		txtTitle = (TextView) findViewById(R.id.txt_select_your_video);

		txtHeading.setTypeface(UtilityClass.getTfHelveticaNueuLtPro(context));
		txtTitle.setTypeface(UtilityClass.getTfHelveticaNueu(context));

		imgBtnSearch = (ImageButton) findViewById(R.id.img_btn_search);
		imgBtnSearch.setOnClickListener(new SearchBtnClickListner(this));

		rltLayoutSearch = (RelativeLayout) findViewById(R.id.rlt_layout_search);
		etSearch = (EditText) findViewById(R.id.et_search);
		imgBtnSearchInspirations = (ImageButton) findViewById(R.id.img_btn_search_inspiration);
		imgBtnSearchInspirations.setOnClickListener(new YouTubeSearchListener(
				this));

		listYoutubeVideos = (PullToRefreshListView) findViewById(R.id.list_video);
		btnMore = new Button(this);
		btnMore.setText("More >>");
		btnMore.setBackgroundDrawable(getResources().getDrawable(
				R.drawable.youtube_row));

		listYoutubeVideos.setOnRefreshListener(new RefreshListListener());

	}

	private class RefreshListListener implements OnRefreshListener {

		@Override
		public void onRefresh() {
			rltLayoutSearch.setVisibility(View.GONE);
			youtubeVedioList = new ArrayList<VideoListModel>();
			thumbnailArrayList = new ArrayList<String>();
			startIndex = 1;
			i = 0;

			String requestUrl = categoryUrl + START_INDEX + startIndex
					+ MAX_RESULT + maxResult + URL_END_TAG + query;

			requestModel.setRequestUrl(requestUrl);

			new ApiCallAsyncTask(YouTubeVideoListActivity.this, null, apiCall,
					requestModel, null, "", "", GET_METHOD)
					.execute((Void) null);
		}

	}

	private class GetVideosListAsynkTask extends AsyncTask<Void, Void, Boolean> {

		private Context context;

		public GetVideosListAsynkTask(Context context) {

			this.context = context;
		}

		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			progressDialog = new CustomProgressDialog(context, "YouTube",
					"Loading, Please Wait..");
			progressDialog.setCancelable(false);
			progressDialog.show();

		}

		@Override
		protected Boolean doInBackground(Void... params) {
			try {

				ArrayList<YouTubeItems> youTubeItems = youTubeData.getData()
						.getItems();

				int size = youTubeItems.size();

				Logger.debug("Items " + size);

				for (int pos = 0; pos < size; pos++) {
					/*
					 * YouTubeThumbnails youTubeThumbnails =
					 * youTubeItems.get(pos) .getThumbnail();
					 */

					String videoId = youTubeItems.get(pos).getId();
					String sqThumbnail = UtilityClass
							.createYoutubeThumbnailfromId(SQ_QUALITY, videoId);
					String hqThumbnail = UtilityClass
							.createYoutubeThumbnailfromId(HQ_QUALITY, videoId);
					String url = UtilityClass
							.createYoutubeWatchUrlFromId(videoId);
					String title = youTubeItems.get(pos).getTitle();
					String date = youTubeItems.get(pos).getUpdated();
					String desc = youTubeItems.get(pos).getDescription();
					String views = youTubeItems.get(pos).getViewCount();
					String duration = youTubeItems.get(pos).getDuration();

					int sec = (Integer.parseInt(duration));

					int mm = (sec / 60);
					int ss = (sec % 60);

					NumberFormat timeFormatter = new DecimalFormat("#00");

					duration = timeFormatter.format(mm);
					duration = duration + ":" + timeFormatter.format(ss);

					String summery = "Views: " + views + " - Duration: "
							+ duration;

					VideoListModel videoListModel = new VideoListModel();
					videoListModel.setSqThumbnail(sqThumbnail);
					videoListModel.setHqThumbnail(hqThumbnail);
					videoListModel.setTitle(title);
					videoListModel.setDate(date);
					videoListModel.setDesc(desc);
					videoListModel.setUrl(url);
					videoListModel.setViews(summery);

					youtubeVedioList.add(videoListModel);
				}

				return true;
			} catch (Exception e) {
				return false;

			}

		}

		@Override
		protected void onPostExecute(Boolean result) {
			// progressDialog.dismiss();
			if (i < youtubeVedioList.size()) {
				new ImageLoadAsynkTask(YouTubeVideoListActivity.this,
						youtubeVedioList.get(i).getSqThumbnail(), context)
						.execute((Void) null);
			}

			/*
			 * videoListAdaptar = new VideoListAdapter(context,
			 * youtubeVedioList); videoListView.setAdapter(videoListAdaptar);
			 * videoListAdaptar.notifyDataSetChanged();
			 * 
			 * Logger.debug("" + youtubeVedioList.size());
			 * 
			 * videoListView .setOnItemClickListener(new
			 * VideoListItemClickListener( context, youtubeVedioList));
			 * Logger.debug("*******ADAPTAR SET*******");
			 */

		}
	}

	@Override
	protected void onPause() {
		super.onPause();
		overridePendingTransition(0, 0);
	}

	@Override
	public void apiCallAsyncTaskResult(String jsonFromPost) {
		listYoutubeVideos.onRefreshComplete();
		progressDialog.dismiss();
		/* youTubeData = new YouTubeModelForJson(); */

		Gson gson = new Gson();
		if (jsonFromPost != null) {
			youTubeData = gson
					.fromJson(jsonFromPost, YouTubeModelForJson.class);
			Logger.debug("Data received");
		}
		Logger.debug("RESPONSE" + jsonFromPost);

		String msg = null;

		YouTubeError youTubeError = youTubeData.getError();

		if (youTubeError != null) {
			msg = youTubeData.getError().getMessage();
		}

		if (msg != null) {
			UtilityClass.showAlerDialog(context, ERROR, "NO Record Found",
					null, 0);
		} else {

			String totalItems = youTubeData.getData().getTotalItems();

			if (totalItems != null) {
				int count = Integer.valueOf(totalItems);

				startIndex = Integer.valueOf(youTubeData.getData()
						.getStartIndex());

				startIndex += Integer.valueOf(youTubeData.getData()
						.getItemsPerPage());

				if (startIndex < count) {
					listYoutubeVideos.addFooterView(btnMore);
				}

				if (count != 0) {
					new GetVideosListAsynkTask(context).execute((Void) null);
				} else {
					UtilityClass.showAlerDialog(context, ERROR,
							"NO Record Found", null, 0);
				}
			} else {
				UtilityClass.showAlerDialog(context, ERROR, "NO Record Found",
						null, 0);
			}
		}

	}

	@Override
	public void imageLoadCompletedWithResult(String localPath) {
		i++;
		thumbnailArrayList.add(localPath);
		if (i < youtubeVedioList.size()) {

			String url = youtubeVedioList.get(i).getSqThumbnail();

			if (url != null)
				new ImageLoadAsynkTask(this, url, context).execute((Void) null);
			else {
				imageLoadCompletedWithResult(null);
			}
		}

		else {

			for (int j = 0; j < thumbnailArrayList.size(); j++) {
				youtubeVedioList.get(j).setImgLocalPath(
						thumbnailArrayList.get(j));

			}
			videoListAdaptar = new VideoListAdapter(context, youtubeVedioList);
			listYoutubeVideos.setAdapter(videoListAdaptar);
			videoListAdaptar.notifyDataSetChanged();

			Logger.debug("" + youtubeVedioList.size());

			listYoutubeVideos
					.setOnItemClickListener(new VideoListItemClickListener(
							context, youtubeVedioList));
			Logger.debug("*******ADAPTAR SET*******");
			progressDialog.dismiss();
		}

		listYoutubeVideos
				.setOnItemClickListener(new VideoListItemClickListener(context,
						youtubeVedioList));

	}

	@Override
	public void onBackPressed() {
		super.onBackPressed();
		finish();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	public void searchClickResult(boolean result) {
		if (rltLayoutSearch.isShown())
			rltLayoutSearch.setVisibility(View.GONE);
		else
			rltLayoutSearch.setVisibility(View.VISIBLE);
	}

	@Override
	public void searchYoutubeVideos(int code) {
		rltLayoutSearch.setVisibility(View.GONE);

		String searchText = etSearch.getText().toString().trim();

		if (searchText.equals("")) {
			UtilityClass.showAlerDialog(context, ERROR,
					"Please enter search text", null, 0);
		} else {

			if (UtilityClass.getNetworkState(context)) {

				youtubeVedioList = new ArrayList<VideoListModel>();
				thumbnailArrayList = new ArrayList<String>();
				startIndex = 1;
				i = 0;

				String toSearch = QUERY_TAG + searchText;

				toSearch = toSearch.replace(" ", "%20");

				String requestUrl = categoryUrl + START_INDEX + startIndex
						+ MAX_RESULT + maxResult + URL_END_TAG + toSearch;

				Logger.debug(requestUrl);

				requestModel.setRequestUrl(requestUrl);

				new ApiCallAsyncTask(this, null, apiCall, requestModel, null,
						SEARCHING_HEADING, SEARCHING_MSG, GET_METHOD)
						.execute((Void) null);
			} else
				UtilityClass.showAlerDialog(context, ERROR,
						INTERNET_NOT_AVAILABLE, null, 0);

		}

	}
}
