package com.mobisoft.youtubedatareading.network;

import android.content.Context;
import android.os.AsyncTask;

import com.mobisoft.youtubedatareading.CustomProgressDialog;
import com.mobisoft.youtubedatareading.interfaces.ApiCallAsyncTaskDelegate;
import com.mobisoft.youtubedatareading.interfaces.KeyValuePairInterface;
import com.mobisoft.youtubedatareading.interfaces.PostViewDelegate;
import com.mobisoft.youtubedatareading.utils.Logger;

public class ApiCallAsyncTask extends AsyncTask<Void, Void, String> implements
		KeyValuePairInterface {

	private APICall apiCall;
	private AbstractModel requestModel;
	private Context context;
	private String jsonPostResponse;
	private CustomProgressDialog progressDialog;
	private ApiCallAsyncTaskDelegate delegate;
	private PostViewDelegate postViewDelegate;
	private String heading;
	private String msg;
	private int method;

	public ApiCallAsyncTask(ApiCallAsyncTaskDelegate delegate,
			PostViewDelegate postViewDelegate, APICall apiCall,
			AbstractModel requestModel, Context context, String heading,
			String msg, int method) {

		this.delegate = delegate;
		this.postViewDelegate = postViewDelegate;
		this.apiCall = apiCall;
		this.requestModel = requestModel;
		this.context = context;
		this.heading = heading;
		this.msg = msg;
		this.method = method;

	}

	@Override
	protected void onPreExecute() {
		super.onPreExecute();
		if (context != null) {
			progressDialog = new CustomProgressDialog(context, heading, msg);
			progressDialog.setCancelable(false);
			progressDialog.show();
		}
	}

	@Override
	protected String doInBackground(Void... params) {

		try {
			if (method == GET_METHOD)
				jsonPostResponse = apiCall.jsonFromUrlGetRequest(requestModel);
			else if (method == POST_METHOD)
				jsonPostResponse = apiCall.jsonFromUrlPostRequest(requestModel);
			else if (method == PUT_METHOD)
				jsonPostResponse = apiCall.jsonFromUrlPutRequest(requestModel);
			else if (method == DELETE_METHOD)
				jsonPostResponse = apiCall
						.jsonFromUrlDeleteRequest(requestModel);

			return jsonPostResponse;
		} catch (Exception net) {
			net.printStackTrace();
			Logger.error("error" + net.getMessage());
			return "";
		}
	}

	@Override
	protected void onPostExecute(String jsonPostResponse) {
		if (context != null)
			progressDialog.dismiss();
		String url = requestModel.getRequestUrl();

		if (postViewDelegate != null) {
			if (url.contains(AbstractModel.POST_DETAIL_URL)) {
				postViewDelegate.postDetailResult(jsonPostResponse);
			}
		}

		if (delegate != null)
			delegate.apiCallAsyncTaskResult(jsonPostResponse);

	}

}
