package com.mobisoft.youtubedatareading.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.BitmapFactory;
import android.graphics.Typeface;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import com.mobisoft.youtubedatareading.R;
import com.mobisoft.youtubedatareading.interfaces.AlertDialogDelegate;
import com.mobisoft.youtubedatareading.interfaces.KeyValuePairInterface;

public class UtilityClass implements KeyValuePairInterface {

	public static Stack<Activity> ACTIVITY_STACK = new Stack<Activity>();

	public static boolean emailValidation(String mailId) {
		Pattern pattern = Pattern.compile(EMAIL_PATTERN);
		Matcher matcher = pattern.matcher(mailId);
		boolean matchFound = matcher.matches();
		return matchFound;
	}

	public static String getTime(String time) {
		String newTime = time;

		try {
			final SimpleDateFormat simpleDateFormat = new SimpleDateFormat(
					"hh:mm:ss");
			final Date dateObject = simpleDateFormat.parse(time);
			Logger.debug(dateObject.toString());
			newTime = new SimpleDateFormat("hh:mm aa").format(dateObject);
			Logger.debug("Formatted Date :" + newTime);

		} catch (final ParseException e) {
			e.printStackTrace();
			newTime = time;
		}

		return newTime;

	}

	public static boolean getNetworkState(Context context) {
		ConnectivityManager connectivityManager = (ConnectivityManager) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		if (connectivityManager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE)
				.getState() == NetworkInfo.State.CONNECTED
				|| connectivityManager.getNetworkInfo(
						ConnectivityManager.TYPE_WIFI).getState() == NetworkInfo.State.CONNECTED) {
			// we are connected to a network
			return true;
		} else
			return false;
	}

	public static byte[] getBytesFromFile(File file) throws IOException {
		byte[] bytes = null;
		DataInputStream dis = null;
		try {

			// File length
			int size = (int) file.length();
			Logger.debug("" + size);
			if (size > Integer.MAX_VALUE) {
				System.out.println("File is to larger");
			}
			bytes = new byte[size];
			dis = new DataInputStream(new FileInputStream(file));
			int read = 0;
			int numRead = 0;
			while (read < bytes.length
					&& (numRead = dis.read(bytes, read, bytes.length - read)) >= 0) {
				read = read + numRead;
			}
			Logger.debug("File size: " + read);
			// Ensure all the bytes have been read in
			if (read < bytes.length) {
				Logger.debug("Could not completely read: " + file.getName());

			}
		} catch (Exception e) {
			e.getMessage();
		} finally {
			if (dis != null) {
				dis.close();
			}
		}

		while (bytes.length > IMAGE_FILE_LIMIT) {
			UtilityClass utilityClass = new UtilityClass();
			Bitmap bitmap = utilityClass.reduceBitmap(bytes);
			ByteArrayOutputStream blob = new ByteArrayOutputStream();
			bitmap.compress(CompressFormat.JPEG, 95 /* ignored for PNG */, blob);
			bytes = blob.toByteArray();
			Logger.debug("While " + bytes.length);

		}
		return bytes;
	}

	public static Bitmap getCacheImageFromSdCard(String imgPath)
			throws IOException {
		UtilityClass utilityClass = new UtilityClass();
		File file = new File(imgPath);
		byte[] bytes = getCacheImageByte(file);
		Bitmap bitmap = utilityClass.cacheBitmap(bytes);
		return bitmap;

	}

	public static byte[] getCacheImageByte(File file) throws IOException {
		byte[] bytes = null;
		DataInputStream dis = null;
		try {

			// File length
			int size = (int) file.length();
			Logger.debug("" + size);
			if (size > Integer.MAX_VALUE) {
				System.out.println("File is to larger");
			}
			bytes = new byte[size];
			dis = new DataInputStream(new FileInputStream(file));
			int read = 0;
			int numRead = 0;
			while (read < bytes.length
					&& (numRead = dis.read(bytes, read, bytes.length - read)) >= 0) {
				read = read + numRead;
			}
			Logger.debug("File size: " + read);
			// Ensure all the bytes have been read in
			if (read < bytes.length) {
				Logger.debug("Could not completely read: " + file.getName());

			}
		} catch (Exception e) {
			e.getMessage();
		} finally {

			if (dis != null) {
				dis.close();
			}

		}

		return bytes;
	}

	public Bitmap cacheBitmap(byte[] fileByteArray) {
		InputStream is = new ByteArrayInputStream(fileByteArray);
		// BitmapFactory.Options options = new BitmapFactory.Options();
		Bitmap bitmapImage = BitmapFactory.decodeStream(is, null, null);
		if (bitmapImage == null)
			Logger.debug("Bitmap is null");

		return bitmapImage;
	}

	public Bitmap reduceBitmap(byte[] fileByteArray) {
		InputStream is = new ByteArrayInputStream(fileByteArray);

		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;
		BitmapFactory.decodeStream(is, null, options);

		BitmapFactory.decodeStream(is, null, options);

		/*
		 * If image size is large reduce loaded image size to avoid out of
		 * memory error.
		 */

		options.inSampleSize = 2;

		/* Keep just in bounds false to load image in component */
		options.inJustDecodeBounds = false;
		is = new ByteArrayInputStream(fileByteArray);
		// Bitmap bitmapImage = BitmapFactory.decodeStream(is);
		Bitmap bitmapImage = BitmapFactory.decodeStream(is, null, options);
		if (bitmapImage == null)
			Logger.debug("Bitmap is null");

		return bitmapImage;
	}

	public static Bitmap requiredBitmapFromFile(String imgPath)
			throws IOException {
		UtilityClass utilityClass = new UtilityClass();
		File file = new File(imgPath);
		byte[] bytes = getBytesFromFile(file);
		Bitmap bitmap = utilityClass.reduceBitmap(bytes);
		return bitmap;

	}

	public static Bitmap loadBitmap(String url) throws IOException {
		Logger.debug("URL  : " + url);
		Bitmap bitmap = null;
		InputStream in = null;

		try {
			in = new java.net.URL(url).openStream();
			bitmap = BitmapFactory.decodeStream(in);

		} catch (Exception e) {
			Logger.error(e.getMessage());
			e.printStackTrace();
		} finally {
			in.close();

		}

		return bitmap;
	}

	public static String getDateByMilliSeconds(String date) {
		long yourmilliseconds = Long.parseLong(date);
		SimpleDateFormat sdf = new SimpleDateFormat("MMM dd,yyyy hh:mm aa");
		Date resultdate = new Date(yourmilliseconds);
		return sdf.format(resultdate);
	}

	public static String getDateForCommentScreen(String date) {
		long yourmilliseconds = Long.parseLong(date);
		SimpleDateFormat sdfDate = new SimpleDateFormat("MMM dd");
		SimpleDateFormat sdTime = new SimpleDateFormat("hh:mm aa");

		Date resultdate = new Date(yourmilliseconds);

		String convertDate = sdfDate.format(resultdate);
		String convertTime = sdTime.format(resultdate);
		convertTime = convertTime.toLowerCase();
		return convertDate + " at " + convertTime;
	}

	public static String getTruncateString(String desc) {
		int length = 15;
		if (desc != null) {
			desc = desc.replaceAll("\\r\\n|\\r|\\n", " ");
			if (desc.length() > length) {
				desc = desc.substring(0, length) + "...";
			}

		}

		return desc;
	}

	public static String getYouTubeIdFromUrl(String videoUrl) {

		try {
			Logger.debug("Video url :" + videoUrl);

			String pattern = "(?:videos\\/|v=)([\\w-]+)";
			String youtubeVideoId = null;
			Pattern compiledPattern = Pattern.compile(pattern);
			Matcher matcher = compiledPattern.matcher(videoUrl);

			if (matcher.find()) {
				youtubeVideoId = matcher.group();

				youtubeVideoId = youtubeVideoId.substring(2,
						youtubeVideoId.length());

				// https://www.youtube.com/v/UdBE1-BiXes?version=3&f=standard&app=youtube_gdata

				String id = youtubeVideoId;

				return id;
			} else {
				youtubeVideoId = videoUrl.substring(26, videoUrl.indexOf("?"));
				Logger.debug("ID" + youtubeVideoId);
				Logger.debug("INDEX" + videoUrl.indexOf("\\v\\"));

				String id = youtubeVideoId;
				return id;
			}
		} catch (Exception e) {
			Logger.debug("Exception");
		}

		return "";

	}

	public static String getYouTubeVideoThumbnailUrl(String videoUrl) {

		try {
			Logger.debug("Video url :" + videoUrl);

			String pattern = "(?:videos\\/|v=)([\\w-]+)";
			String youtubeVideoId = null;
			Pattern compiledPattern = Pattern.compile(pattern);
			Matcher matcher = compiledPattern.matcher(videoUrl);

			if (matcher.find()) {
				youtubeVideoId = matcher.group();

				youtubeVideoId = youtubeVideoId.substring(2,
						youtubeVideoId.length());

				// https://www.youtube.com/v/UdBE1-BiXes?version=3&f=standard&app=youtube_gdata

				String url = "http://img.youtube.com/vi/" + youtubeVideoId
						+ "/default.jpg";

				return url;
			} else {
				youtubeVideoId = videoUrl.substring(26, videoUrl.indexOf("?"));
				Logger.debug("ID" + youtubeVideoId);
				Logger.debug("INDEX" + videoUrl.indexOf("\\v\\"));

				String url = "http://img.youtube.com/vi/" + youtubeVideoId
						+ "/default.jpg";
				return url;
			}
		} catch (Exception e) {
			Logger.debug("Exception");
		}

		return "";

	}

	public static Typeface getTfHelveticaNueu(Context context) {
		Typeface tf = Typeface.createFromAsset(context.getAssets(),
				"font/HelveticaNeue.otf");
		return tf;
	}

	public static Typeface getTfHelveticaNueuLtPro(Context context) {
		Typeface tf = Typeface.createFromAsset(context.getAssets(),
				"font/HelveticaNeueLTPro.otf");
		return tf;
	}

	public static void showAlerDialog(Context context, String title,
			String msg, AlertDialogDelegate delegate, int code) {
		AlertDialog.Builder alertDialog = new AlertDialog.Builder(context);
		alertDialog.setTitle(title);
		alertDialog.setMessage(msg);
		alertDialog.setCancelable(false);
		alertDialog.setIcon(context.getResources().getDrawable(
				R.drawable.error_icon));
		alertDialog.setPositiveButton("OK", new AlertDialogListenter(delegate,
				code));
		alertDialog.show();

	}

	public static void showAlerDialogRefreshCancel(Context context,
			String title, String msg, AlertDialogDelegate delegate, int code) {

		AlertDialog.Builder alertDialog = new AlertDialog.Builder(context);
		alertDialog.setTitle(title);
		alertDialog.setMessage(msg);
		alertDialog.setCancelable(false);
		alertDialog.setIcon(context.getResources().getDrawable(
				R.drawable.error_icon));
		alertDialog.setPositiveButton("Refresh", new AlertDialogListenter(
				delegate, code));
		alertDialog.setNegativeButton("Cancel", new AlerDialogCancelListener());
		alertDialog.show();

	}

	public static void showAlerDialogOkCancel(Context context, String title,
			String msg, AlertDialogDelegate delegate, int code1, int code2) {

		AlertDialog.Builder alertDialog = new AlertDialog.Builder(context);
		alertDialog.setTitle(title);
		alertDialog.setMessage(msg);
		alertDialog.setCancelable(false);
		alertDialog.setIcon(context.getResources().getDrawable(
				R.drawable.error_icon));
		alertDialog.setPositiveButton("Ok", new AlertDialogListenter(delegate,
				code1));
		alertDialog.setNegativeButton("Cancel", new AlertDialogListenter(
				delegate, code2));
		alertDialog.show();

	}

	public static void createAppDataFolderInSDCard() {
		new File(APP_DATA_PATH).mkdirs();
		try {
			File filename = new File(APP_DATA_PATH + ".nomedia");
			FileOutputStream out = new FileOutputStream(filename);

			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public static String createYoutubeThumbnailfromId(String quality,
			String youtubeVideoId) {
		String thumbnailUrl = "http://img.youtube.com/vi/" + youtubeVideoId
				+ "/" + quality + "default.jpg";
		return thumbnailUrl;
	}

	public static String createYoutubeWatchUrlFromId(String videoId) {
		String watchUrl = "http://www.youtube.com/watch?v=" + videoId;
		return watchUrl;
	}

	public static void deleteAppFolder(File file) throws IOException {

		if (file.isDirectory()) {

			// directory is empty, then delete it
			if (file.list().length == 0) {

				file.delete();
				System.out.println("Directory is deleted : "
						+ file.getAbsolutePath());

			} else {

				// list all the directory contents
				String files[] = file.list();

				for (String temp : files) {
					// construct the file structure
					File fileDelete = new File(file, temp);

					// recursive delete
					deleteAppFolder(fileDelete);
				}

				// check the directory again, if empty then delete it
				if (file.list().length == 0) {
					file.delete();
					System.out.println("Directory is deleted : "
							+ file.getAbsolutePath());
				}
			}

		} else {
			// if file, then delete it
			file.delete();
			System.out.println("File is deleted : " + file.getAbsolutePath());
		}
	}

}
